#include <QtGui/QPainter>
#include <algorithm>
#include "imageDisplay.hh"


using namespace mimas;
using namespace std;

#define RADIANS ( 2.0 * M_PI / 360.0 )
#define STEPSIZE 1.0
#define TEXTURESIZE 512


ImageDisplay::ImageDisplay( QWidget *parent, const QGLWidget *shareWidget,
                            Qt::WindowFlags f ):
  QGLWidget( parent, shareWidget, f ), m_aspect(1.0), hfov( 5.0 ),
  pfov( 90.0 ), currentImage(0), flashing(false)
{
}

void ImageDisplay::setFlashing( bool on )
{
  if ( on )
    m_timer = startTimer( 300 );
  else {
    killTimer( m_timer );
    flashing = false;
    update();
  };
}

void ImageDisplay::timerEvent( QTimerEvent *event )
{
  flashing = !flashing;
  update();
}

/*
void ImageDisplay::recompute(void)
{
  const int pWidth = 800, pHeight = 600;
  const double pAlpha = 80.0 * 2.0 * M_PI / 360.0;
  image< rgba< unsigned char > > img2; img2.init( pWidth, pHeight );	
  cerr << "recomputing for " << m_images.size() << " image(s)" << endl;

  for ( int k=0; k<m_images.size(); k++ ) {

// Yaw Rotation Matrix
   cerr << "(yaw,pitch,roll) = (" << ( ypr[k].yaw * 180.0 / M_PI ) << "," << ( ypr[k].pitch * 180.0 / M_PI ) << "," << ( ypr[k].roll * 180.0 / M_PI ) << ")" << endl;
   Matrix yawMatrix( 3, 3 );

         yawMatrix( 0, 0 )= cos(ypr[k].yaw);
	 yawMatrix( 0, 1 )= 0;
         yawMatrix( 0, 2 )= -sin(ypr[k].yaw);
         yawMatrix( 1, 0 )= 0;
	 yawMatrix( 1, 1 )= 1;
         yawMatrix( 1, 2 )= 0;
	 yawMatrix( 2, 0 )= sin(ypr[k].yaw);
	 yawMatrix( 2, 1 )= 0;
	 yawMatrix( 2, 2 )= cos(ypr[k].yaw);

// Pitch rotation Matrix

  Matrix pitchMatrix( 3, 3 );
	 
	 pitchMatrix( 0, 0 )= 1;
	 pitchMatrix( 0, 1 )= 0;
	 pitchMatrix( 0, 2 )= 0;
	 pitchMatrix( 1, 0 )= 0;
	 pitchMatrix( 1, 1 )= cos(ypr[k].pitch);
	 pitchMatrix( 1, 2 )= sin(ypr[k].pitch);
	 pitchMatrix( 2, 0 )= 0;
	 pitchMatrix( 2, 1 )= -sin(ypr[k].pitch);
	 pitchMatrix( 2, 2 )= cos(ypr[k].pitch);


// Roll Rotation Matrix


  Matrix rollMatrix( 3, 3 );
        
         
         rollMatrix( 0, 0 ) = cos(ypr[k].roll);
         rollMatrix( 0, 1 ) = sin (ypr[k].roll);
         rollMatrix( 0, 2 )= 0;
         rollMatrix( 1, 0 )= -sin(ypr[k].roll);
	 rollMatrix( 1, 1 )= cos(ypr[k].roll);
         rollMatrix( 1, 2 )= 0;
	 rollMatrix( 2, 0 )= 0;
         rollMatrix( 2, 1 )= 0;
	 rollMatrix( 2, 2 )= 1;




// boost::numeric::ublas::identity_matrix< double > id( 3 );
   Matrix tempMatrix( prod( pitchMatrix, yawMatrix ) ); 
   Matrix rotation( prod(  tempMatrix, rollMatrix ) );
   // Matrix rotation( id );

   cerr << ( k + 1 ) << " - the image has size " << m_images[k].getWidth() << "x" << m_images[k].getHeight() << endl;

   
   for (int j=0; j<pHeight; j++) {
     for (int i=0; i<pWidth; i++) {
	Vector v( 3 );
	v[1] = j - pHeight  / 2;
	v[0] = i - pWidth / 2;
	v[2] = ( 0.5 * pWidth ) / tan(pAlpha/2);
        Vector vs( prod( rotation, v ) );
        int
	  xs = (int)( vs[ 0 ] / vs[ 2 ] * ( 0.5 * m_images[k].getWidth() ) / tan(hfov/2) ) + pWidth  / 2,
	  ys = (int)( vs[ 1 ] / vs[ 2 ] * ( 0.5 * m_images[k].getWidth() ) / tan(hfov/2) ) + pHeight / 2;
	rgba< unsigned char > c;

        if ( xs >= 0 && ys >= 0 && xs < m_images[k].getWidth() && ys < m_images[k].getHeight() ) {
	   c = m_images[k].pixel( xs, ys );
           img2.pixel( i, j ) = c;
	}
      }
    }
  };

  // convert from mimas::image to QImage.
  QImage temp( img2.getWidth(), img2.getHeight(), QImage::Format_RGB32 );

  // Line-wise copying required, as QImage does 32 bit memory alignment!
  // Direct access to image-memory!
  const rgba< unsigned char > *p = img2.rawData();
  for( int y=0; y<img2.getHeight(); y++ ) {
    copy( p, p + img2.getWidth(), (rgba< unsigned char > *)temp.scanLine( y ) );
    p += img2.getWidth();
  };

  qImg = temp;
  update();
} */

ImageDisplay::~ImageDisplay(void)
{
  glDeleteTextures( MAXSIZE, m_texName );
}

void ImageDisplay::addImage(const image< rgba< unsigned char > > &_img, double yaw, double pitch, double roll )
{
  // img.init( _img.getWidth(), _img.getHeight() );
  m_images.push_back( _img );
  YPR _ypr( yaw, pitch, roll);
  ypr.push_back( _ypr );  
  // recompute();
  // update();
};

void ImageDisplay::setHFov( double _hfov )
{
  hfov = _hfov;
}

void ImageDisplay::setPFov( double _pfov )
{
  pfov = _pfov;
  update();
}

void ImageDisplay:: clearImages(){
  m_images.clear();
  m_textures.clear();
  ypr.clear();
  currentImage = 0;
}

/*
void ImageDisplay::paintEvent( QPaintEvent * )
{
  QPainter p( this );
  p.drawImage( 0, 0, qImg );
}*/


void ImageDisplay::setYaw( double _yaw){
  if ( currentImage < ypr.size() ) {
    ypr[currentImage].yaw = _yaw;
    update();
  }
}

void ImageDisplay::setPitch( double _pitch){
  if ( currentImage < ypr.size() ) {
    ypr[currentImage].pitch = _pitch;
    update();
  }
}

void ImageDisplay::setRoll( double _roll){
  if ( currentImage < ypr.size() ) {
    ypr[currentImage].roll = _roll;
    update();
  }
}


void ImageDisplay:: setCurrentImage(int _currentImage){
   if ( _currentImage >= 0 && _currentImage < m_images.size() ) {
     currentImage = _currentImage;
     emit currentImageChanged( currentImage );
     emit yawChanged( ypr[ currentImage ].yaw );
     emit pitchChanged( ypr[ currentImage ].pitch);
     emit rollChanged( ypr[ currentImage ].roll);
   };
}

void ImageDisplay::keyPressEvent( QKeyEvent *event )
{
  switch ( event->key() ) {
  case Qt::Key_Left:
#ifndef NDEBUG
    cerr << "Left" << endl;
#endif
    if ( currentImage < ypr.size()) {
      ypr[currentImage].yaw -= STEPSIZE;
      emit yawChanged( ypr[ currentImage ].yaw );
    };
    update();
    break;
  case Qt::Key_Right:
#ifndef NDEBUG
    cerr << "Right" << endl;
#endif
    if ( currentImage < ypr.size() ) {
      ypr[currentImage].yaw += STEPSIZE;
      emit yawChanged( ypr[ currentImage ].yaw );
    }
    update();
    break;
  case Qt::Key_Up:
#ifndef NDEBUG
    cerr << "Up" << endl;
#endif
    if ( currentImage < ypr.size() ) {
      ypr[currentImage].pitch += STEPSIZE;
      emit pitchChanged( ypr[ currentImage ].pitch);
    }
    update();
    break;
  case Qt::Key_Down:
#ifndef NDEBUG
    cerr << "Down" << endl;
#endif
    if ( currentImage < ypr.size() ) {
      ypr[currentImage].pitch -= STEPSIZE;
      emit pitchChanged( ypr[ currentImage ].pitch);
    }
    update();
    break;
  case Qt::Key_End:
#ifndef NDEBUG
    cerr << "End" << endl;
#endif
    if ( currentImage < ypr.size() ) {
      ypr[currentImage].roll -= STEPSIZE;
      emit rollChanged( ypr[ currentImage ].roll);
    }
    update();
    break;
  case Qt::Key_PageDown:
#ifndef NDEBUG
    cerr << "PageDown" << endl;
#endif
    if ( currentImage < ypr.size() ) {
      ypr[currentImage].roll += STEPSIZE;
      emit rollChanged( ypr[ currentImage ].roll);
    }
    update();
    break;
  case Qt::Key_1:
#ifndef NDEBUG
    cerr << "Selecting image 0" << endl;
#endif
    setCurrentImage( 0 );
    break;
  case Qt::Key_2:
#ifndef NDEBUG
    cerr << "Selecting image 1" << endl;
#endif
    setCurrentImage( 1 );
    break;
  case Qt::Key_3:
#ifndef NDEBUG
    cerr << "Selecting image 2" << endl;
#endif
    setCurrentImage( 2 );
    break;
  case Qt::Key_4:
#ifndef NDEBUG
    cerr << "Selecting image 3" << endl;
#endif
    setCurrentImage( 3 );
    break;
  case Qt::Key_5:
#ifndef NDEBUG
    cerr << "Selecting image 4" << endl;
#endif
    setCurrentImage( 4 );
    break;
  case Qt::Key_6:
#ifndef NDEBUG
    cerr << "Selecting image 5" << endl;
#endif
    setCurrentImage( 5 );
    break;
  case Qt::Key_7:
#ifndef NDEBUG
    cerr << "Selecting image 6" << endl;
#endif
    setCurrentImage( 6 );
    break;
  case Qt::Key_8:
#ifndef NDEBUG
    cerr << "Selecting image 7" << endl;
#endif
    setCurrentImage( 7 );
    break;
  case Qt::Key_9:
#ifndef NDEBUG
    cerr << "Selecting image 8" << endl;
#endif
    setCurrentImage( 8 );
    break;
  case Qt::Key_0:
#ifndef NDEBUG
    cerr << "Selecting image 9" << endl;
#endif
    setCurrentImage( 9 );
    break;
  default:
    QGLWidget::keyPressEvent( event );
  };
}


void ImageDisplay::initializeGL(void)
{
#ifndef NDEBUG
  cerr << "initializeGL" << endl;
#endif
  // qglClearColor( Qt::white );
  qglClearColor( Qt::black );
  // qglClearColor( Qt::gray );
  glShadeModel( GL_FLAT );
  glEnable( GL_BLEND );
  // glEnable( GL_DEPTH_TEST );

  glGenTextures( MAXSIZE, m_texName );

}

void ImageDisplay::resizeGL( int w, int h )
{
#ifndef NDEBUG
  cerr << "resizeGL" << endl;
#endif
  glViewport( 0, 0, (GLint)w, (GLint)h );
  m_aspect = (GLfloat)w / (GLfloat)h;
}

void ImageDisplay::paintGL(void)
{
#ifndef NDEBUG
  cerr << "paintGL" << endl;
#endif

  while ( m_textures.size() < m_images.size() ) {
    image< rgba< unsigned char > > texture;
    texture.init( TEXTURESIZE, TEXTURESIZE );
    int i = m_textures.size();
    assert( i < MAXSIZE );
    int h = TEXTURESIZE * m_images[i].getHeight() / m_images[i].getWidth();
    assert( h <= TEXTURESIZE );
    for ( int y=0; y<h; y++ )
      for ( int x=0; x<TEXTURESIZE; x++ ) {
        texture.pixel( x, y ) =
          m_images[i].pixel( x * m_images[i].getWidth() / TEXTURESIZE,
                             y * m_images[i].getWidth() / TEXTURESIZE );
        texture.pixel( x, y ).setAlpha( 192 );
      };
    m_textures.push_back( texture );
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glBindTexture( GL_TEXTURE_2D, m_texName[i] );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
                    GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_LINEAR );
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, TEXTURESIZE,
                  TEXTURESIZE, 0, GL_BGRA, GL_UNSIGNED_BYTE,
                  m_textures[i].rawData() );
  };


  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
#ifndef NDEBUG
  cerr << "  m_aspect = " << m_aspect << endl;
#endif
  float d = 1.0 / tan( hfov * RADIANS / 2.0 );

  gluPerspective( pfov, m_aspect, 0.01, 2.0 * d );

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glEnable( GL_TEXTURE_2D );
    
  for ( int i=0; i<m_images.size(); i++ ) {
    if ( i != currentImage || flashing )
      glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    else
      glBlendFunc( GL_ONE, GL_ONE_MINUS_SRC_ALPHA );
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    
    glRotatef( ypr[i].yaw, 0.0, -1.0, 0.0 );
    glRotatef( ypr[i].pitch, 1.0, 0.0, 0.0 );
    glRotatef( ypr[i].roll, 0.0, 0.0, -1.0 );
    
#ifndef NDEBUG
    cerr << "  distance = " << d << endl;
#endif
    glTranslatef( 0.0, 0.0, -d );
    
#ifndef NDEBUG
    cerr << "  drawing texture " << m_texName[i] << " with ( "
         << ypr[i].yaw << ", " << ypr[i].pitch << ", " << ypr[i].roll << ")" << endl;
#endif
    glBindTexture( GL_TEXTURE_2D, m_texName[i] );
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT );
    float h = (float)m_images[i].getHeight() / m_images[i].getWidth();
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0); glVertex3f( -1.0,  h, 0.0 );
    glTexCoord2f(0.0, h  ); glVertex3f( -1.0, -h, 0.0 );
    glTexCoord2f(1.0, h  ); glVertex3f(  1.0, -h, 0.0 );
    glTexCoord2f(1.0, 0.0); glVertex3f(  1.0,  h, 0.0 );
    glEnd();
  };
  glDisable( GL_TEXTURE_2D );
  glFlush();
}
