#ifndef __IMAGEDISPLAY_HH
#define __IMAGEDISPLAY_HH

#include <mimas/image.h>
#include <mimas/rgba.h>
#include <QtGui/QWidget>
#include <QtGui/QImage>
#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <fstream>
#include <mimas/image_fileinput.h>
#include <GL/glut.h>
#include <QtGui/QApplication>
#include <QtOpenGL/QGLWidget>
#include <QtGui/QColor>
#include <QtGui/QKeyEvent>
#include <vector>

#define MAXSIZE 16

class YPR {
public:
   YPR( double _yaw, double _pitch, double _roll ): yaw(_yaw), pitch(_pitch), roll(_roll) {}
   double yaw;
   double pitch;
   double roll; 
};

class ImageDisplay: public QGLWidget
{
  Q_OBJECT
public:
  /// Type-definition for convenience.
  typedef boost::numeric::ublas::matrix< double > Matrix;
  /// Type-definition for convenience.
  typedef boost::numeric::ublas::vector< double > Vector;
  typedef boost::numeric::ublas::vector< double > ImageVector;
  ImageDisplay( QWidget *parent = 0, const QGLWidget *shareWidget = 0,
                Qt::WindowFlags f=0 );
  virtual ~ImageDisplay(void);
  int numImages(void) { return m_images.size(); }
  const std::vector< YPR > &getYpr() const { return ypr; }
signals:
  void yawChanged(double);
  void pitchChanged(double);
  void rollChanged(double);
  void pfovChanged(double);
  void currentImageChanged(int);
public slots:
  // void recompute();
  void addImage ( const mimas::image< mimas::rgba< unsigned char > > &_img, double yaw, double pitch, double roll );
  void setHFov( double _hfov );
  void setPFov( double _pfov );
  // void showImages ( const mimas::image< mimas::rgba< unsigned char > > &_images );
  void setYaw(double);
  void setPitch(double);
  void setRoll(double);
  void setFlashing(bool);
  void setCurrentImage(int);
  void clearImages();
  void initializeGL(void);
  void paintGL(void);
  void resizeGL( int w, int h );
protected:
  virtual void timerEvent( QTimerEvent *event );
  virtual void keyPressEvent( QKeyEvent *event );
  // virtual void paintEvent( QPaintEvent * );
  QImage qImg;
  std::vector< mimas::image< mimas::rgba< unsigned char > > > m_images;
  std::vector< mimas::image< mimas::rgba< unsigned char > > > m_textures;
  GLuint m_texName[MAXSIZE];
  GLfloat m_aspect;
//Aqi tengo q crear otro igual para mostrar todas las imagenes en panorama
  std::vector< YPR > ypr;
  double hfov;
  double pfov;
  int currentImage;
  int m_timer;
  bool flashing;
};

#endif
