#include <fstream>
#include <mimas/image_fileinput.h>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include "panoramaWindow.hh"
#include <iostream>
#include <fstream>
#include <sstream>

#define RADIANS ( 2.0 * M_PI / 360.0 )

using namespace mimas;
using namespace std;



/*typedef struct YPR {
   double yaw;
   double pitch;
   double roll; 
};


typedef struct ControlPoints {
   double x;
   double y
   double index;
};

std::vector<YPR>angles;
std::vector<std::pair<ControlPoint, ControlPoint>>Control;
*/



PanoramaWindow::PanoramaWindow( QWidget *parent, Qt::WFlags f ):
  QWidget( parent, f )
{
  ui.setupUi( this );
  connect( ui.closeButton, SIGNAL(clicked()), this, SLOT(close()) );

  connect( ui.yawbutton, SIGNAL(valueChanged(double)), ui.display, SLOT(setYaw(double)));
  connect( ui.pitchbutton, SIGNAL(valueChanged(double)), ui.display, SLOT(setPitch(double)));
  connect( ui.rollbutton, SIGNAL(valueChanged(double)), ui.display, SLOT(setRoll(double)));
  connect( ui.pfovbutton, SIGNAL(valueChanged(double)), ui.display, SLOT(setPFov(double)));

  connect( ui.display, SIGNAL(yawChanged(double)), ui.yawbutton, SLOT(setValue(double)));
  connect( ui.display, SIGNAL(pitchChanged(double)), ui.pitchbutton, SLOT(setValue(double)));
  connect( ui.display, SIGNAL(rollChanged(double)), ui.rollbutton, SLOT(setValue(double)));
  connect( ui.display, SIGNAL(pfovChanged(double)), ui.pfovbutton, SLOT(setValue(double)));

  connect( ui.imageProperties, SIGNAL(clicked()), this, SLOT(readpto()));
  connect( ui.indexSpinBox, SIGNAL(valueChanged(int)), ui.display, SLOT(setCurrentImage(int)));
  connect( ui.display, SIGNAL(currentImageChanged(int)), ui.indexSpinBox, SLOT(setValue(int)));

  connect( ui.flashCheckBox, SIGNAL(toggled(bool)), ui.display, SLOT(setFlashing(bool)) );
}



void PanoramaWindow::loadImage(void)
{
  QString fileName =
    QFileDialog::getOpenFileName( this, "Load Image", QString(),
                                  "Images (*.png *.jpg *.bmp);;All Files (*)" );
  if ( !fileName.isEmpty() ) {

    try {

      ifstream f( fileName.toLatin1(), ios::binary );
      image< rgba< unsigned char > > img;
      MMERROR( f >> img, mimasexception, ,
               "Error loading file \"" << (const char *)fileName.toLatin1()
               << "\"." );
      //ui.display->addImage( img, yaw, pitch, roll, _hfov );
    } catch ( exception &e ) {
      
      QMessageBox::critical( this, "Error loading image",
                             e.what() );

    };

  };
};

/*void PanoramaWindow::rotateroll( double d )
{
  cerr << "Roll" << endl;
  cerr << d << endl;
  ui.display->setRoll( d * M_PI / 180.0 );
}

void PanoramaWindow::rotatepitch( double d )
{
  cerr << "Pitch" << endl;
  cerr << d << endl;
  ui.display->setPitch( d * M_PI / 180.0 );
}*/


//Read the pto file


void PanoramaWindow:: readpto(){

   
   string str;
   std::vector<std::string> line;
//OPEN PTO FILE

    QString fileName =
    QFileDialog::getOpenFileName( this, "Load Pto File", QString(),
                                  "Pto Files (*.pto);;All Files (*)" );
    if ( !fileName.isEmpty() ) {

       ifstream f( fileName.toLatin1() ); 

       ui.display->clearImages();

       while(f) {
	
          string cadena;
          getline( f, cadena, '\n');


     
// IDENTIFY IMAGE


       if ( cadena[0] == 'i' ) {

         int pos = cadena.rfind( "n\"" );
         cout << endl << endl << "IMAGE NAME--> " << cadena.substr( pos + 2, cadena.length() - pos - 3 ) << endl << endl;
        
	 //Opening the image identified image

	 string fileName = cadena.substr( pos + 2, cadena.length() - pos - 3 );

         ifstream file( fileName.c_str(), ios::binary );
         image< rgba< unsigned char > > img;
         MMERROR( file >> img, mimasexception, ,
                 "Error loading file \"" << fileName.c_str()
                 << "\"." );

         istringstream stream( cadena );

         double yaw = 0.0, pitch = 0.0, roll = 0.0;

         for ( int i=0; i<27; i++ ) {
           stream >> str;
           switch ( i ) {
           case 22:
	     cerr << "Pitch = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             pitch = atof( str.substr( 1, str.length() - 1 ).c_str() ); 
             break;

           case 26:
             cerr << "Yaw = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             yaw = atof( str.substr( 1, str.length() - 1 ).c_str() );
             break;

           case 23:
             cerr << "Roll = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             roll = atof( str.substr( 1, str.length() - 1 ).c_str() ) ;
             break;

           case 25:
             
	     double hfov=atof( str.substr( 1, str.length() - 1 ).c_str() ); 
             //Control that the hfov is > 0 
             if (hfov>0){ 
                  cerr << "HFOV--> " << hfov << endl;
                  ui.display->setHFov( hfov ); 
                  // getchar();
             }
          }
	 }
           ui.display->addImage( img, yaw, pitch, roll );
           
       }

       


        if( cadena[0] == 'c' ){
        
        istringstream stream( cadena );
	// double x1, x2, y1, y2;
 
        for ( int c=0; c<6; c++ ) {
           stream >> str;
           
	  switch ( c ) {
           case 3:
	     cerr << "Control Point X1 = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             break;

           case 4:
             cerr << "Control Point Y1 = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             break;

           case 5:
             cerr << "Control Point X2 = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             break;

           case 6:
             cerr << "Control Point Y2 = " << atof( str.substr( 1, str.length() - 1 ).c_str() ) << endl;
             break;
           }
         }
        
        }   
        // getchar();
     
   }


        ui.indexSpinBox->setMaximum( ui.display->numImages() - 1 );
        ui.indexSpinBox->setValue( 0 );
        ui.yawbutton->setValue( ui.display->getYpr()[0].yaw );
        ui.pitchbutton->setValue( ui.display->getYpr()[0].pitch );
        ui.rollbutton->setValue( ui.display->getYpr()[0].roll );
        // ui.display->recompute();
        ui.display->update();
    }
}






